/**
 * @file
 * Shadow volumes class definition.
 *
 * @author Tom "Shamot" Burian
 */
/* ---------------------------------------------------------------------------
 * Authors:      Tom Burian (tburian _AT centrum.cz)
 * Thanks to:    PCJohn (peciva _AT fit.vutbr.cz)
 * Contributors:
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 * ---------------------------------------------------------------------------
 */

#ifndef SO_SHADOW_VOLUME__H
#define SO_SHADOW_VOLUME__H

//----------------------------------------------------------------------------
// INCLUDEs
//----------------------------------------------------------------------------

#include "ShadowManager.h"
#include "ObjectModel.h"
#include "SoShadowCaster.h"
#include "SoShadowLight.h"
//
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLight.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoMatrixTransform.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoTriangleStripSet.h>
#include <Inventor/nodes/SoTransform.h> 
//
#include <Inventor/SbLinear.h>
#include <Inventor/SoOutput.h>
//
#include <Inventor/fields/SoSFVec3f.h>
#include <Inventor/fields/SoMFVec3f.h>
//
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/actions/SoGetMatrixAction.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/actions/SoCallbackAction.h>
//
#include <Inventor/misc/SoChildList.h>

//----------------------------------------------------------------------------
// IMPLEMENTATION
//----------------------------------------------------------------------------

/**
 * Class defining a shadow volume of one scene object for one light.
 *
 * This "shadow volume" consists of a number of shadow volumes created
 * for every shadow casting triangle. 
 */
class SoShadowVolume : public SoSeparator
{
  private:
    /** Pointer to shadow light. */
    SoShadowLight * shadowlight;
    /** Which light creates this shadow volume. */
    SoLight * aLight;
    /** Object model. */
    SoShadowCaster * caster;
    /** Object model. */
    CObjectModel * objectModel;
    /** Transformation matrix of the object. */
    SbMatrix * trMatrix;
    /** SV coordinates. */
    SoCoordinate3 * coordNode;
    /** Volume color. */
    SbColor svColor;
    /** Transform node. */
    SoTransform * transform; 
    /** index name of caster that casts this volume. */
    int casterName;
    /** Node of shadow volumes material and translation settings. */
    SoGroup * initNode;
    /** Node of shadow volumes of light-facing triangles. */
    SoGroup * volumeNode;

    void init();

  public:
    /** Number of shadow casting triangles of the object. */
    int32_t nScTr;
    /** Number of triangles of the shadow volume. */
    int32_t nSVTr;
    
    SoShadowVolume(SoShadowLight * l, int cn);
    ~SoShadowVolume();
    
    void create();
    int getCasterName();
    void writeShadowVolume(const char *filename);
    
//    static SbVec3f farPoint(SbVec3f lightPos, SbVec3f point);
    static SbVec3f faceNormal(const SbVec3f p1, const SbVec3f p2, const SbVec3f p3);
    static SbVec3f farPointNew(SbVec3f lightVec, SbVec3f point, float farDepth);
    
};

#endif
